#ifndef APP_BLE_DISS_H
#define APP_BLE_DISS_H

#include <stdint.h>

#include "rwip_config.h"  // SW Configuration

/* Define ------------------------------------------------------------*/

/// Manufacturer Name Value
#define APP_DISS_MANUFACTURER_NAME     ("Nations")
#define APP_DISS_MANUFACTURER_NAME_LEN (7)

/// Model Number String Value
#define APP_DISS_MODEL_NB_STR     ("NS-BLE-1.0")
#define APP_DISS_MODEL_NB_STR_LEN (10)

/// Serial Number
#define APP_DISS_SERIAL_NB_STR     ("1.0.0.0-LE")
#define APP_DISS_SERIAL_NB_STR_LEN (10)

/// Firmware Revision
#define APP_DISS_FIRM_REV_STR     ("1.0.0")
#define APP_DISS_FIRM_REV_STR_LEN (5)

/// System ID Value - LSB -> MSB
#define APP_DISS_SYSTEM_ID     ("\x12\x34\x56\xFF\xFE\x9A\xBC\xDE")
#define APP_DISS_SYSTEM_ID_LEN (8)

/// Hardware Revision String
#define APP_DISS_HARD_REV_STR     ("1.0.0")
#define APP_DISS_HARD_REV_STR_LEN (5)

/// Software Revision String
#define APP_DISS_SW_REV_STR     ("1.0.0")
#define APP_DISS_SW_REV_STR_LEN (5)

/// IEEE
#define APP_DISS_IEEE     ("\xFF\xEE\xDD\xCC\xBB\xAA")
#define APP_DISS_IEEE_LEN (6)

/**
 * PNP ID Value - LSB -> MSB
 *      Vendor ID Source : 0x02 (USB Implementer’s Forum assigned Vendor ID value)
 *      Vendor ID : 0x045E      (Microsoft Corp)
 *      Product ID : 0x0040
 *      Product Version : 0x0300
 */
#define APP_DISS_PNP_ID     ("\x02\x5E\x04\x40\x00\x00\x03")
#define APP_DISS_PNP_ID_LEN (7)

#if (BLE_APP_HID)
#define APP_DISS_FEATURES                                                                              \
    (DIS_MANUFACTURER_NAME_CHAR_SUP_BIT | DIS_MODEL_NB_STR_CHAR_SUP_BIT | DIS_SYSTEM_ID_CHAR_SUP_BIT | \
     DIS_PNP_ID_CHAR_SUP_BIT)

#else
#define APP_DISS_FEATURES (DIS_ALL_FEAT_SUP)
#endif  //(BLE_APP_HID)

void app_ble_diss_init(void);

#endif  // APP_BLE_DISS_H
