#include "app_ble_profiles/app_ble_bass.h"

/* Stack configuration */
#include "rwip_config.h"

/* BLE Stack */
#include "ns_ble.h"

/* Profile */
#include "bass.h"
#include "bass_task.h"

static int app_ble_bass_level_upd_handler(ke_msg_id_t const msgid, struct bass_batt_level_upd_rsp const* param,
                                          ke_task_id_t const dest_id, ke_task_id_t const src_id);

static int app_ble_bass_level_ntf_cfg_ind_handler(ke_msg_id_t const msgid, void const* param,
                                                  ke_task_id_t const dest_id, ke_task_id_t const src_id);

static int app_ble_bass_default_handler(ke_msg_id_t const msgid, void const* param, ke_task_id_t const dest_id,
                                        ke_task_id_t const src_id);

static const struct ke_msg_handler app_ble_bass_msg_handler_list[] = {
    {.id = KE_MSG_DEFAULT_HANDLER, .func = (ke_msg_func_t)app_ble_bass_default_handler},
    {.id = BASS_BATT_LEVEL_UPD_REQ, .func = (ke_msg_func_t)app_ble_bass_level_upd_handler},
    {.id = BASS_BATT_LEVEL_NTF_CFG_IND, .func = (ke_msg_func_t)app_ble_bass_level_ntf_cfg_ind_handler},
};

static const struct app_subtask_handlers app_ble_bass_handlers = APP_HANDLERS(app_ble_bass);

void app_ble_bass_init(void) {
    struct bass_db_cfg*               db_cfg;
    struct gapm_profile_task_add_cmd* req = KE_MSG_ALLOC_DYN(GAPM_PROFILE_TASK_ADD_CMD, TASK_GAPM, TASK_APP,
                                                             gapm_profile_task_add_cmd, sizeof(struct bass_db_cfg));
    req->operation                        = GAPM_PROFILE_TASK_ADD;
    req->sec_lvl                          = PERM(SVC_AUTH, NO_AUTH);
    req->prf_task_id                      = TASK_ID_BASS;
    req->app_task                         = TASK_APP;
    req->start_hdl                        = 0;

    db_cfg = (struct bass_db_cfg*)req->param;

    db_cfg->bas_nb      = 1;
    db_cfg->features[0] = BAS_BATT_LVL_NTF_SUP;

    ke_msg_send(req);

    struct prf_task_t prf;
    prf.prf_task_id      = TASK_ID_BASS;
    prf.prf_task_handler = &app_ble_bass_handlers;
    ns_ble_prf_task_register(&prf);

    struct prf_get_func_t get_func;
    get_func.task_id          = TASK_ID_BASS;
    get_func.prf_itf_get_func = bass_prf_itf_get;
    prf_get_itf_func_register(&get_func);
}

void app_ble_bass_enable_prf(uint8_t conidx) {
    struct bass_enable_req* req =
        KE_MSG_ALLOC(BASS_ENABLE_REQ, prf_get_task_from_id(TASK_ID_BASS), TASK_APP, bass_enable_req);

    req->conidx          = conidx;
    req->ntf_cfg         = PRF_CLI_STOP_NTFIND;
    req->old_batt_lvl[0] = 100;

    ke_msg_send(req);
}

void app_ble_bass_update_level(uint8_t level) {
    struct bass_batt_level_upd_req* req =
        KE_MSG_ALLOC(BASS_BATT_LEVEL_UPD_REQ, prf_get_task_from_id(TASK_ID_BASS), TASK_APP, bass_batt_level_upd_req);

    req->bas_instance = 0;
    req->batt_level   = level;

    ke_msg_send(req);
}

static int app_ble_bass_level_upd_handler(ke_msg_id_t const msgid, struct bass_batt_level_upd_rsp const* param,
                                          ke_task_id_t const dest_id, ke_task_id_t const src_id) {
    return KE_MSG_CONSUMED;
}

static int app_ble_bass_level_ntf_cfg_ind_handler(ke_msg_id_t const msgid, void const* param,
                                                  ke_task_id_t const dest_id, ke_task_id_t const src_id) {
    return KE_MSG_CONSUMED;
}

static int app_ble_bass_default_handler(ke_msg_id_t const msgid, void const* param, ke_task_id_t const dest_id,
                                        ke_task_id_t const src_id) {
    return KE_MSG_CONSUMED;
}