/* FreeRTOS */
#include "FreeRTOS.h"
#include "task.h"

/* Device */
#include "n32wb03x.h"

/* Board */
#include "board.h"

/* Profiles */
#include "app_ble_profiles/app_ble_bass.h"

/* App */
#include "app_ble_support.h"

static void task_initialize(void *parameters);

int main(void) {
    board_led_init(0);
    board_led_init(1);

    if (app_ble_init() < 0) {
        goto dead_loop;
    }

    if (xTaskCreate(task_initialize, "INIT", 256, NULL, 4, NULL) != pdTRUE) {
        goto dead_loop;
    }

    vTaskStartScheduler();

dead_loop:
    for (;;) {
        /* -- */
        __WFI();
    }
}

static void task_initialize(void *parameters) {
    uint8_t battery_level = 0U;
    for (;;) {
        if (app_ble_lock(100) >= 0) {
            board_led_set(0, true);
            app_ble_bass_update_level(battery_level);
            battery_level++;
            if (battery_level > 100) {
                battery_level = 0;
            }

            board_led_set(0, false);
            app_ble_unlock();
        }

        vTaskDelay(pdMS_TO_TICKS(5000));
    }
}